@@ -193,7 +193,7 @@
 
 /* This is also defined in stddef.h */
 #ifndef offsetof
-#define offsetof(TYPE, MEMBER) ((size_t)&((TYPE *)0)->MEMBER)
+#define offsetof(TYPE, MEMBER) ((size_t) & ((TYPE *)0)->MEMBER)
 #endif
 
 /*****************************************************************************/
@@ -663,17 +663,6 @@ static inline void *_nl_memdup(const void *ptr, size_t len)
 
 /*****************************************************************************/
 
-static inline size_t _nl_addr_family_to_size(int addr_family)
-{
-	if (addr_family == AF_INET)
-		return sizeof(in_addr_t);
-	if (addr_family == AF_INET6)
-		return sizeof(struct in6_addr);
-	return 0;
-}
-
-/*****************************************************************************/
-
 typedef union {
 	in_addr_t addr4;
 	struct in_addr a4;
@@ -709,40 +698,6 @@ static inline char *_nl_inet_ntop_dup(int addr_family, const void *addr)
 						    INET6_ADDRSTRLEN));
 }
 
-static inline char *_nl_inet_ntop4(in_addr_t addr,
-				   char buf[static INET_ADDRSTRLEN])
-{
-	return _nl_inet_ntop(AF_INET, &addr, buf);
-}
-
-static inline char *_nl_inet_ntop6(const struct in6_addr *addr,
-				   char buf[static INET6_ADDRSTRLEN])
-{
-	return _nl_inet_ntop(AF_INET6, &addr, buf);
-}
-
-/*****************************************************************************/
-
-static inline size_t _nl_ptrarray_len_(const void *const *ptr, ssize_t len)
-{
-	size_t l = 0;
-
-	if (len >= 0)
-		return len;
-	if (ptr) {
-		while (ptr[l])
-			l++;
-	}
-	return l;
-}
-
-#define _nl_ptrarray_len(ptr, len)                                   \
-	({                                                           \
-		typeof(*(ptr)) *const _ptr = (ptr);                  \
-                                                                     \
-		_nl_ptrarray_len_((const void *const *)_ptr, (len)); \
-	})
-
 /*****************************************************************************/
 
 #define _NL_AUTO_DEFINE_FCN_VOID0(CastType, name, func) \
@@ -769,32 +724,11 @@ static inline size_t _nl_ptrarray_len_(const void *const *ptr, ssize_t len)
 	}                                                \
 	struct _nl_dummy_for_tailing_semicolon
 
-#define _NL_AUTO_DEFINE_FCN_INDIRECT0(CastType, name, func) \
-	static inline void name(CastType *v)                \
-	{                                                   \
-		if (*v)                                     \
-			func(v);                            \
-	}                                                   \
-	struct _nl_dummy_for_tailing_semicolon
-
 #define _nl_auto_free _nl_auto(_nl_auto_free_fcn)
 _NL_AUTO_DEFINE_FCN_VOID0(void *, _nl_auto_free_fcn, free);
 
 /*****************************************************************************/
 
-#define _nl_swap(p_a, p_b)                          \
-	do {                                        \
-		typeof(*(p_a)) *const _p_a = (p_a); \
-		typeof(*(p_a)) *const _p_b = (p_b); \
-		typeof(*(p_a)) _tmp;                \
-                                                    \
-		_tmp = *_p_a;                       \
-		*_p_a = *_p_b;                      \
-		*_p_b = _tmp;                       \
-	} while (0)
-
-/*****************************************************************************/
-
 #define NSEC_PER_SEC 1000000000L
 
 struct trans_tbl {
@@ -802,7 +736,10 @@ struct trans_tbl {
 	const char *a;
 };
 
-#define __ADD(id, name) { .i = id, .a = #name }
+#define __ADD(id, name)             \
+	{                           \
+		.i = id, .a = #name \
+	}
 
 #define BUG()                                                                \
 	do {                                                                 \
@@ -861,8 +798,8 @@ static inline void nl_write_unlock(pthread_rwlock_t *lock)
 }
 
 #else
-#define NL_LOCK(NAME) int __unused_lock_##NAME _nl_unused
-#define NL_RW_LOCK(NAME) int __unused_lock_##NAME _nl_unused
+#define NL_LOCK(NAME) int __unused_lock_##NAME __attribute__((unused))
+#define NL_RW_LOCK(NAME) int __unused_lock_##NAME __attribute__((unused))
 
 #define nl_lock(LOCK) \
 	do {          \
