@@ -25,7 +25,6 @@
 #include <netlink/route/cls/ematch.h>
 
 #include "tc-api.h"
-#include "nl-aux-route/nl-route.h"
 
 struct rtnl_basic
 {
@@ -224,10 +223,12 @@ int rtnl_basic_add_action(struct rtnl_cls *cls, struct rtnl_act *act)
 	if (!(b = rtnl_tc_data(TC_CAST(cls))))
 		return -NLE_NOMEM;
 
-	if ((err = _rtnl_act_append_get(&b->b_act, act)) < 0)
+	b->b_mask |= BASIC_ATTR_ACTION;
+	if ((err = rtnl_act_append(&b->b_act, act)))
 		return err;
 
-	b->b_mask |= BASIC_ATTR_ACTION;
+	/* In case user frees it */
+	rtnl_act_get(act);
 	return 0;
 }
 
