@@ -15,7 +15,6 @@
 #include <netlink/route/cls/flower.h>
 
 #include "tc-api.h"
-#include "nl-aux-route/nl-route.h"
 
 /** @cond SKIP */
 struct rtnl_flower {
@@ -810,7 +809,6 @@ int rtnl_flower_get_ipv4_dst(struct rtnl_cls *cls, in_addr_t *out_addr,
 int rtnl_flower_append_action(struct rtnl_cls *cls, struct rtnl_act *act)
 {
 	struct rtnl_flower *f;
-	int err;
 
 	if (!act)
 		return 0;
@@ -818,11 +816,10 @@ int rtnl_flower_append_action(struct rtnl_cls *cls, struct rtnl_act *act)
 	if (!(f = rtnl_tc_data(TC_CAST(cls))))
 		return -NLE_NOMEM;
 
-	if ((err = _rtnl_act_append_get(&f->cf_act, act)) < 0)
-		return err;
-
 	f->cf_mask |= FLOWER_ATTR_ACTION;
-	return 0;
+
+	rtnl_act_get(act);
+	return rtnl_act_append(&f->cf_act, act);
 }
 
 /**
