@@ -54,8 +54,6 @@
 #include "nl-priv-dynamic-core/object-api.h"
 #include "nl-priv-dynamic-core/nl-core.h"
 #include "nl-priv-dynamic-core/cache-api.h"
-#include "nl-aux-core/nl-core.h"
-#include "nl-aux-xfrm/nl-xfrm.h"
 
 /** @cond SKIP */
 
@@ -109,7 +107,6 @@ struct xfrmnl_sa {
 	uint32_t                        seq;
 	uint32_t                        reqid;
 	uint16_t                        family;
-	uint32_t                        if_id;
 	uint8_t                         mode;        /* XFRM_MODE_xxx */
 	uint8_t                         replay_window;
 	uint8_t                         flags;
@@ -158,7 +155,6 @@ struct xfrmnl_sa {
 #define XFRM_SA_ATTR_REPLAY_STATE   0x2000000
 #define XFRM_SA_ATTR_EXPIRE         0x4000000
 #define XFRM_SA_ATTR_OFFLOAD_DEV    0x8000000
-#define XFRM_SA_ATTR_IF_ID          0x10000000
 
 static struct nl_cache_ops  xfrmnl_sa_ops;
 static struct nl_object_ops xfrm_sa_obj_ops;
@@ -362,7 +358,6 @@ static uint64_t xfrm_sa_compare(struct nl_object *_a, struct nl_object *_b,
 	diff |= _DIFF(XFRM_SA_ATTR_COADDR, nl_addr_cmp(a->coaddr, b->coaddr));
 	diff |= _DIFF(XFRM_SA_ATTR_MARK,
 		      (a->mark.m != b->mark.m) || (a->mark.v != b->mark.v));
-	diff |= _DIFF(XFRM_SA_ATTR_IF_ID, a->if_id != b->if_id);
 	diff |= _DIFF(XFRM_SA_ATTR_SECCTX,
 		      ((a->sec_ctx->ctx_doi != b->sec_ctx->ctx_doi) ||
 		       (a->sec_ctx->ctx_alg != b->sec_ctx->ctx_alg) ||
@@ -442,7 +437,6 @@ static const struct trans_tbl sa_attrs[] = {
 	__ADD(XFRM_SA_ATTR_REPLAY_STATE, replay_state),
 	__ADD(XFRM_SA_ATTR_EXPIRE, expire),
 	__ADD(XFRM_SA_ATTR_OFFLOAD_DEV, user_offload),
-	__ADD(XFRM_SA_ATTR_IF_ID, if_id),
 };
 
 static char* xfrm_sa_attrs2str(int attrs, char *buf, size_t len)
@@ -626,9 +620,6 @@ static void xfrm_sa_dump_line(struct nl_object *a, struct nl_dump_params *p)
 	if (sa->ce_mask & XFRM_SA_ATTR_MARK)
 		nl_dump_line(p, "\tMark mask: 0x%x Mark value: 0x%x\n", sa->mark.m, sa->mark.v);
 
-	if (sa->ce_mask & XFRM_SA_ATTR_IF_ID)
-		nl_dump_line(p, "\tXFRM interface ID: 0x%x\n", sa->if_id);
-
 	if (sa->ce_mask & XFRM_SA_ATTR_SECCTX)
 		nl_dump_line(p, "\tDOI: %d Algo: %d Len: %u ctx: %s\n", sa->sec_ctx->ctx_doi,
 		             sa->sec_ctx->ctx_alg, sa->sec_ctx->ctx_len, sa->sec_ctx->ctx);
@@ -770,7 +761,6 @@ static struct nla_policy xfrm_sa_policy[XFRMA_MAX+1] = {
 	[XFRMA_MARK]            = { .minlen = sizeof(struct xfrm_mark) },
 	[XFRMA_TFCPAD]          = { .type = NLA_U32 },
 	[XFRMA_REPLAY_ESN_VAL]  = { .minlen = sizeof(struct xfrm_replay_state_esn) },
-	[XFRMA_IF_ID]           = { .type = NLA_U32 },
 };
 
 static int xfrm_sa_request_update(struct nl_cache *c, struct nl_sock *h)
@@ -780,17 +770,18 @@ static int xfrm_sa_request_update(struct nl_cache *c, struct nl_sock *h)
 
 int xfrmnl_sa_parse(struct nlmsghdr *n, struct xfrmnl_sa **result)
 {
-	_nl_auto_nl_addr struct nl_addr *addr1 = NULL;
-	_nl_auto_nl_addr struct nl_addr *addr2 = NULL;
-	_nl_auto_xfrmnl_sa struct xfrmnl_sa *sa = NULL;
+	struct xfrmnl_sa*           sa;
 	struct nlattr               *tb[XFRMA_MAX + 1];
 	struct xfrm_usersa_info*    sa_info;
 	struct xfrm_user_expire*    ue;
 	int                         len, err;
+	struct nl_addr*             addr;
 
 	sa = xfrmnl_sa_alloc();
-	if (!sa)
-		return -NLE_NOMEM;
+	if (!sa) {
+		err = -NLE_NOMEM;
+		goto errout;
+	}
 
 	sa->ce_msgtype = n->nlmsg_type;
 	if (n->nlmsg_type == XFRM_MSG_EXPIRE)
@@ -811,18 +802,26 @@ int xfrmnl_sa_parse(struct nlmsghdr *n, struct xfrmnl_sa **result)
 
 	err = nlmsg_parse(n, sizeof(struct xfrm_usersa_info), tb, XFRMA_MAX, xfrm_sa_policy);
 	if (err < 0)
-		return err;
+		goto errout;
 
-	if (!(addr1 = _nl_addr_build(sa_info->sel.family, &sa_info->sel.daddr)))
-		return -NLE_NOMEM;
-	nl_addr_set_prefixlen (addr1, sa_info->sel.prefixlen_d);
-	xfrmnl_sel_set_daddr (sa->sel, addr1);
+	if (sa_info->sel.family == AF_INET)
+		addr    = nl_addr_build (sa_info->sel.family, &sa_info->sel.daddr.a4, sizeof (sa_info->sel.daddr.a4));
+	else
+		addr    = nl_addr_build (sa_info->sel.family, &sa_info->sel.daddr.a6, sizeof (sa_info->sel.daddr.a6));
+	nl_addr_set_prefixlen (addr, sa_info->sel.prefixlen_d);
+	xfrmnl_sel_set_daddr (sa->sel, addr);
+	/* Drop the reference count from the above set operation */
+	nl_addr_put(addr);
 	xfrmnl_sel_set_prefixlen_d (sa->sel, sa_info->sel.prefixlen_d);
 
-	if (!(addr2 = _nl_addr_build(sa_info->sel.family, &sa_info->sel.saddr)))
-		return -NLE_NOMEM;
-	nl_addr_set_prefixlen (addr2, sa_info->sel.prefixlen_s);
-	xfrmnl_sel_set_saddr (sa->sel, addr2);
+	if (sa_info->sel.family == AF_INET)
+		addr    = nl_addr_build (sa_info->sel.family, &sa_info->sel.saddr.a4, sizeof (sa_info->sel.saddr.a4));
+	else
+		addr    = nl_addr_build (sa_info->sel.family, &sa_info->sel.saddr.a6, sizeof (sa_info->sel.saddr.a6));
+	nl_addr_set_prefixlen (addr, sa_info->sel.prefixlen_s);
+	xfrmnl_sel_set_saddr (sa->sel, addr);
+	/* Drop the reference count from the above set operation */
+	nl_addr_put(addr);
 	xfrmnl_sel_set_prefixlen_s (sa->sel, sa_info->sel.prefixlen_s);
 
 	xfrmnl_sel_set_dport (sa->sel, ntohs(sa_info->sel.dport));
@@ -835,14 +834,18 @@ int xfrmnl_sa_parse(struct nlmsghdr *n, struct xfrmnl_sa **result)
 	xfrmnl_sel_set_userid (sa->sel, sa_info->sel.user);
 	sa->ce_mask             |= XFRM_SA_ATTR_SEL;
 
-	if (!(sa->id.daddr = _nl_addr_build(sa_info->family, &sa_info->id.daddr)))
-		return -NLE_NOMEM;
+	if (sa_info->family == AF_INET)
+		sa->id.daddr        = nl_addr_build (sa_info->family, &sa_info->id.daddr.a4, sizeof (sa_info->id.daddr.a4));
+	else
+		sa->id.daddr        = nl_addr_build (sa_info->family, &sa_info->id.daddr.a6, sizeof (sa_info->id.daddr.a6));
 	sa->id.spi              = ntohl(sa_info->id.spi);
 	sa->id.proto            = sa_info->id.proto;
 	sa->ce_mask             |= (XFRM_SA_ATTR_DADDR | XFRM_SA_ATTR_SPI | XFRM_SA_ATTR_PROTO);
 
-	if (!(sa->saddr = _nl_addr_build(sa_info->family, &sa_info->saddr)))
-		return -NLE_NOMEM;
+	if (sa_info->family == AF_INET)
+		sa->saddr           = nl_addr_build (sa_info->family, &sa_info->saddr.a4, sizeof (sa_info->saddr.a4));
+	else
+		sa->saddr           = nl_addr_build (sa_info->family, &sa_info->saddr.a6, sizeof (sa_info->saddr.a6));
 	sa->ce_mask             |= XFRM_SA_ATTR_SADDR;
 
 	sa->lft->soft_byte_limit    =   sa_info->lft.soft_byte_limit;
@@ -878,30 +881,36 @@ int xfrmnl_sa_parse(struct nlmsghdr *n, struct xfrmnl_sa **result)
 
 	if (tb[XFRMA_ALG_AEAD]) {
 		struct xfrm_algo_aead* aead = nla_data(tb[XFRMA_ALG_AEAD]);
-
 		len = sizeof (struct xfrmnl_algo_aead) + ((aead->alg_key_len + 7) / 8);
 		if ((sa->aead = calloc (1, len)) == NULL)
-			return -NLE_NOMEM;
+		{
+			err = -NLE_NOMEM;
+			goto errout;
+		}
 		memcpy ((void *)sa->aead, (void *)aead, len);
 		sa->ce_mask     |= XFRM_SA_ATTR_ALG_AEAD;
 	}
 
 	if (tb[XFRMA_ALG_AUTH_TRUNC]) {
 		struct xfrm_algo_auth* auth = nla_data(tb[XFRMA_ALG_AUTH_TRUNC]);
-
 		len = sizeof (struct xfrmnl_algo_auth) + ((auth->alg_key_len + 7) / 8);
 		if ((sa->auth = calloc (1, len)) == NULL)
-			return -NLE_NOMEM;
+		{
+			err = -NLE_NOMEM;
+			goto errout;
+		}
 		memcpy ((void *)sa->auth, (void *)auth, len);
 		sa->ce_mask     |= XFRM_SA_ATTR_ALG_AUTH;
 	}
 
 	if (tb[XFRMA_ALG_AUTH] && !sa->auth) {
 		struct xfrm_algo* auth = nla_data(tb[XFRMA_ALG_AUTH]);
-
 		len = sizeof (struct xfrmnl_algo_auth) + ((auth->alg_key_len + 7) / 8);
 		if ((sa->auth = calloc (1, len)) == NULL)
-			return -NLE_NOMEM;
+		{
+			err = -NLE_NOMEM;
+			goto errout;
+		}
 		strcpy(sa->auth->alg_name, auth->alg_name);
 		memcpy(sa->auth->alg_key, auth->alg_key, (auth->alg_key_len + 7) / 8);
 		sa->auth->alg_key_len = auth->alg_key_len;
@@ -910,36 +919,43 @@ int xfrmnl_sa_parse(struct nlmsghdr *n, struct xfrmnl_sa **result)
 
 	if (tb[XFRMA_ALG_CRYPT]) {
 		struct xfrm_algo* crypt = nla_data(tb[XFRMA_ALG_CRYPT]);
-
 		len = sizeof (struct xfrmnl_algo) + ((crypt->alg_key_len + 7) / 8);
 		if ((sa->crypt = calloc (1, len)) == NULL)
-			return -NLE_NOMEM;
+		{
+			err = -NLE_NOMEM;
+			goto errout;
+		}
 		memcpy ((void *)sa->crypt, (void *)crypt, len);
 		sa->ce_mask     |= XFRM_SA_ATTR_ALG_CRYPT;
 	}
 
 	if (tb[XFRMA_ALG_COMP]) {
 		struct xfrm_algo* comp = nla_data(tb[XFRMA_ALG_COMP]);
-
 		len = sizeof (struct xfrmnl_algo) + ((comp->alg_key_len + 7) / 8);
 		if ((sa->comp = calloc (1, len)) == NULL)
-			return -NLE_NOMEM;
+		{
+			err = -NLE_NOMEM;
+			goto errout;
+		}
 		memcpy ((void *)sa->comp, (void *)comp, len);
 		sa->ce_mask     |= XFRM_SA_ATTR_ALG_COMP;
 	}
 
 	if (tb[XFRMA_ENCAP]) {
 		struct xfrm_encap_tmpl* encap = nla_data(tb[XFRMA_ENCAP]);
-
 		len = sizeof (struct xfrmnl_encap_tmpl);
 		if ((sa->encap = calloc (1, len)) == NULL)
-			return -NLE_NOMEM;
+		{
+			err = -NLE_NOMEM;
+			goto errout;
+		}
 		sa->encap->encap_type   =   encap->encap_type;
 		sa->encap->encap_sport  =   ntohs(encap->encap_sport);
 		sa->encap->encap_dport  =   ntohs(encap->encap_dport);
-		if (!(sa->encap->encap_oa = _nl_addr_build(sa_info->family,
-							   &encap->encap_oa)))
-			return -NLE_NOMEM;
+		if (sa_info->family == AF_INET)
+			sa->encap->encap_oa =   nl_addr_build (sa_info->family, &encap->encap_oa.a4, sizeof (encap->encap_oa.a4));
+		else
+			sa->encap->encap_oa =   nl_addr_build (sa_info->family, &encap->encap_oa.a6, sizeof (encap->encap_oa.a6));
 		sa->ce_mask     |= XFRM_SA_ATTR_ENCAP;
 	}
 
@@ -949,15 +965,21 @@ int xfrmnl_sa_parse(struct nlmsghdr *n, struct xfrmnl_sa **result)
 	}
 
 	if (tb[XFRMA_COADDR]) {
-		if (!(sa->coaddr = _nl_addr_build(
-			      sa_info->family, nla_data(tb[XFRMA_COADDR]))))
-			return -NLE_NOMEM;
+		if (sa_info->family == AF_INET)
+		{
+			sa->coaddr  = nl_addr_build(sa_info->family, nla_data(tb[XFRMA_COADDR]),
+			                            sizeof (uint32_t));
+		}
+		else
+		{
+			sa->coaddr  = nl_addr_build(sa_info->family, nla_data(tb[XFRMA_COADDR]),
+			                            sizeof (uint32_t) * 4);
+		}
 		sa->ce_mask         |= XFRM_SA_ATTR_COADDR;
 	}
 
 	if (tb[XFRMA_MARK]) {
 		struct xfrm_mark* m =   nla_data(tb[XFRMA_MARK]);
-
 		sa->mark.m  =   m->m;
 		sa->mark.v  =   m->v;
 		sa->ce_mask |= XFRM_SA_ATTR_MARK;
@@ -965,10 +987,12 @@ int xfrmnl_sa_parse(struct nlmsghdr *n, struct xfrmnl_sa **result)
 
 	if (tb[XFRMA_SEC_CTX]) {
 		struct xfrm_user_sec_ctx* sec_ctx = nla_data(tb[XFRMA_SEC_CTX]);
-
 		len = sizeof (struct xfrmnl_user_sec_ctx) + sec_ctx->ctx_len;
 		if ((sa->sec_ctx = calloc (1, len)) == NULL)
-			return -NLE_NOMEM;
+		{
+			err = -NLE_NOMEM;
+			goto errout;
+		}
 		memcpy (sa->sec_ctx, sec_ctx, len);
 		sa->ce_mask     |= XFRM_SA_ATTR_SECCTX;
 	}
@@ -985,10 +1009,12 @@ int xfrmnl_sa_parse(struct nlmsghdr *n, struct xfrmnl_sa **result)
 
 	if (tb[XFRMA_REPLAY_ESN_VAL]) {
 		struct xfrm_replay_state_esn* esn = nla_data (tb[XFRMA_REPLAY_ESN_VAL]);
-
 		len =   sizeof (struct xfrmnl_replay_state_esn) + (sizeof (uint32_t) * esn->bmp_len);
 		if ((sa->replay_state_esn = calloc (1, len)) == NULL)
-			return -NLE_NOMEM;
+		{
+			err = -NLE_NOMEM;
+			goto errout;
+		}
 		memcpy ((void *)sa->replay_state_esn, (void *)esn, len);
 		sa->ce_mask |= XFRM_SA_ATTR_REPLAY_STATE;
 	}
@@ -1006,28 +1032,31 @@ int xfrmnl_sa_parse(struct nlmsghdr *n, struct xfrmnl_sa **result)
 		struct xfrm_user_offload *offload;
 
 		len = sizeof(struct xfrmnl_user_offload);
-		if ((sa->user_offload = calloc(1, len)) == NULL)
-			return -NLE_NOMEM;
+
+		if ((sa->user_offload = calloc(1, len)) == NULL) {
+			err = -NLE_NOMEM;
+			goto errout;
+		}
+
 		offload = nla_data(tb[XFRMA_OFFLOAD_DEV]);
 		sa->user_offload->ifindex = offload->ifindex;
 		sa->user_offload->flags = offload->flags;
 		sa->ce_mask |= XFRM_SA_ATTR_OFFLOAD_DEV;
 	}
 
-	if (tb[XFRMA_IF_ID]) {
-		sa->if_id = nla_get_u32(tb[XFRMA_IF_ID]);
-		sa->ce_mask |= XFRM_SA_ATTR_IF_ID;
-	}
-
-	*result = _nl_steal_pointer(&sa);
+	*result = sa;
 	return 0;
+
+errout:
+	xfrmnl_sa_put(sa);
+	return err;
 }
 
 static int xfrm_sa_update_cache (struct nl_cache *cache, struct nl_object *obj,
                                  change_func_t change_cb, change_func_v2_t change_cb_v2,
 				 void *data)
 {
-	_nl_auto_nl_object struct nl_object* old_sa = NULL;
+	struct nl_object*       old_sa;
 	struct xfrmnl_sa*       sa = (struct xfrmnl_sa*)obj;
 
 	if (nl_object_get_msgtype (obj) == XFRM_MSG_EXPIRE)
@@ -1063,7 +1092,7 @@ static int xfrm_sa_update_cache (struct nl_cache *cache, struct nl_object *obj,
 				else if (change_cb)
 					change_cb(cache, obj, NL_ACT_NEW, data);
 			}
-			else
+			else if (old_sa)
 			{
 				uint64_t diff = 0;
 				if (change_cb || change_cb_v2)
@@ -1077,6 +1106,7 @@ static int xfrm_sa_update_cache (struct nl_cache *cache, struct nl_object *obj,
 					} else if (change_cb)
 						change_cb(cache, obj, NL_ACT_CHANGE, data);
 				}
+				nl_object_put (old_sa);
 			}
 		}
 		else
@@ -1087,6 +1117,7 @@ static int xfrm_sa_update_cache (struct nl_cache *cache, struct nl_object *obj,
 				change_cb_v2(cache, obj, NULL, 0, NL_ACT_DEL, data);
 			else if (change_cb)
 				change_cb (cache, obj, NL_ACT_DEL, data);
+			nl_object_put (old_sa);
 		}
 
 		/* Done handling expire message */
@@ -1378,10 +1409,6 @@ static int build_xfrm_sa_message(struct xfrmnl_sa *tmpl, int cmd, int flags, str
 		offload->flags = tmpl->user_offload->flags;
 	}
 
-	if (tmpl->ce_mask & XFRM_SA_ATTR_IF_ID) {
-		NLA_PUT_U32 (msg, XFRMA_IF_ID, tmpl->if_id);
-	}
-
 	*result = msg;
 	return 0;
 
@@ -1717,27 +1744,6 @@ int xfrmnl_sa_set_family (struct xfrmnl_sa* sa, unsigned int family)
 	return 0;
 }
 
-int xfrmnl_sa_get_if_id (struct xfrmnl_sa* sa, uint32_t* if_id)
-{
-	if (if_id == NULL)
-		return -NLE_INVAL;
-
-	if (!(sa->ce_mask & XFRM_SA_ATTR_IF_ID))
-		return -NLE_NOATTR;
-
-	*if_id = sa->if_id;
-
-	return 0;
-}
-
-int xfrmnl_sa_set_if_id (struct xfrmnl_sa* sa, uint32_t if_id)
-{
-	sa->if_id = if_id;
-	sa->ce_mask |= XFRM_SA_ATTR_IF_ID;
-
-	return 0;
-}
-
 int xfrmnl_sa_get_mode (struct xfrmnl_sa* sa)
 {
 	if (sa->ce_mask & XFRM_SA_ATTR_MODE)
@@ -1862,17 +1868,20 @@ int xfrmnl_sa_set_aead_params (struct xfrmnl_sa* sa, const char* alg_name, unsig
  */
 int xfrmnl_sa_get_auth_params (struct xfrmnl_sa* sa, char* alg_name, unsigned int* key_len, unsigned int* trunc_len, char* key)
 {
-	if (!(sa->ce_mask & XFRM_SA_ATTR_ALG_AUTH))
-		return -NLE_MISSING_ATTR;
+	if (sa->ce_mask & XFRM_SA_ATTR_ALG_AUTH)
+	{
+		if (alg_name)
+			strcpy (alg_name, sa->auth->alg_name);
+		if (key_len)
+			*key_len = sa->auth->alg_key_len;
+		if (trunc_len)
+			*trunc_len = sa->auth->alg_trunc_len;
+		if (key)
+			memcpy (key, sa->auth->alg_key, (sa->auth->alg_key_len + 7)/8);
+	}
+	else
+		return -1;
 
-	if (alg_name)
-		strcpy(alg_name, sa->auth->alg_name);
-	if (key_len)
-		*key_len = sa->auth->alg_key_len;
-	if (trunc_len)
-		*trunc_len = sa->auth->alg_trunc_len;
-	if (key)
-		memcpy(key, sa->auth->alg_key, (sa->auth->alg_key_len + 7) / 8);
 	return 0;
 }
 
@@ -2408,12 +2417,12 @@ static struct nl_cache_ops xfrmnl_sa_ops = {
  * @{
  */
 
-static void _nl_init xfrm_sa_init(void)
+static void __attribute__ ((constructor)) xfrm_sa_init(void)
 {
 	nl_cache_mngt_register(&xfrmnl_sa_ops);
 }
 
-static void _nl_exit xfrm_sa_exit(void)
+static void __attribute__ ((destructor)) xfrm_sa_exit(void)
 {
 	nl_cache_mngt_unregister(&xfrmnl_sa_ops);
 }
